@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2008 The PHP Group                                |
+  | Copyright (c) 1997-2009 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -52,6 +52,10 @@ function_entry geoip_functions[] = {
 	PHP_FE(geoip_db_avail,	NULL)
 	PHP_FE(geoip_db_get_all_info,	NULL)
 	PHP_FE(geoip_db_filename,	NULL)
+#if LIBGEOIP_VERSION >= 1004001
+	PHP_FE(geoip_region_name_by_code,	NULL)
+	PHP_FE(geoip_time_zone_by_country_and_region,	NULL)
+#endif
 	{NULL, NULL, NULL}
 };
 /* }}} */
@@ -165,9 +169,13 @@ PHP_RSHUTDOWN_FUNCTION(geoip)
  */
 PHP_MINFO_FUNCTION(geoip)
 {
+	char buf[32];
+	
 	php_info_print_table_start();
 	php_info_print_table_header(2, "geoip support", "enabled");
 	php_info_print_table_row(2, "geoip extension version", PHP_GEOIP_VERSION);
+	snprintf(buf, sizeof(buf), "%d", LIBGEOIP_VERSION);
+	php_info_print_table_row(2, "geoip library version", buf);
 	php_info_print_table_end();
 	DISPLAY_INI_ENTRIES();
 }
@@ -399,7 +407,7 @@ PHP_FUNCTION(geoip_org_by_name)
 {
 	GeoIP * gi;
 	char * hostname = NULL;
-	const char * org;
+	char * org;
 	int arglen;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
@@ -419,7 +427,8 @@ PHP_FUNCTION(geoip_org_by_name)
 		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
-	RETURN_STRING((char*)org, 1);
+	RETVAL_STRING(org, 1);
+	free(org);
 }
 /* }}} */
 
@@ -456,6 +465,9 @@ PHP_FUNCTION(geoip_record_by_name)
 	}
 	
 	array_init(return_value);
+#if LIBGEOIP_VERSION >= 1004003
+	add_assoc_string(return_value, "continent_code", (gir->continent_code == NULL) ? "" : gir->continent_code, 1);
+#endif
 	add_assoc_string(return_value, "country_code", (gir->country_code == NULL) ? "" : gir->country_code, 1);
 	add_assoc_string(return_value, "country_code3", (gir->country_code3 == NULL) ? "" : gir->country_code3, 1);
 	add_assoc_string(return_value, "country_name", (gir->country_name == NULL) ? "" : gir->country_name, 1);
@@ -464,9 +476,13 @@ PHP_FUNCTION(geoip_record_by_name)
 	add_assoc_string(return_value, "postal_code", (gir->postal_code == NULL) ? "" : gir->postal_code, 1);
 	add_assoc_double(return_value, "latitude", gir->latitude);
 	add_assoc_double(return_value, "longitude", gir->longitude);
+#if LIBGEOIP_VERSION >= 1004005
+	add_assoc_long(return_value, "dma_code", gir->metro_code);
+#else
 	add_assoc_long(return_value, "dma_code", gir->dma_code);
+#endif
 	add_assoc_long(return_value, "area_code", gir->area_code);
-	
+
 	GeoIPRecord_delete(gir);
 }
 /* }}} */
@@ -543,7 +559,7 @@ PHP_FUNCTION(geoip_isp_by_name)
 {
 	GeoIP * gi;
 	char * hostname = NULL;
-	const char * isp;
+	char * isp;
 	int arglen;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hostname, &arglen) == FAILURE) {
@@ -563,9 +579,67 @@ PHP_FUNCTION(geoip_isp_by_name)
 		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Host %s not found", hostname);
 		RETURN_FALSE;
 	}
-	RETURN_STRING((char*)isp, 1);
+	RETVAL_STRING(isp, 1);
+	free(isp);
 }
 
+#if LIBGEOIP_VERSION >= 1004001
+/* {{{ proto string geoip_region_name_by_code( string country_code, string region_code )
+   Returns the region name for some country code and region code combo */
+PHP_FUNCTION(geoip_region_name_by_code)
+{
+	GeoIP * gi;
+	char * country_code = NULL;
+	char * region_code = NULL;
+	const char * region_name;
+	int countrylen, regionlen;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &country_code, &countrylen, &region_code, &regionlen) == FAILURE) {
+		return;
+	}
+
+	if (!countrylen || !regionlen) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "You need to specify the country and region codes.");
+		RETURN_FALSE;
+	}
+	
+	region_name = GeoIP_region_name_by_code(country_code, region_code);
+	if (region_name == NULL) {
+		RETURN_FALSE;
+	}
+	RETURN_STRING((char*)region_name, 1);
+}
+/* }}} */
+#endif
+
+#if LIBGEOIP_VERSION >= 1004001
+/* {{{ proto string geoip_time_zone_by_country_and_region( string country, string region )
+   Returns the time zone for some country code and region code combo */
+PHP_FUNCTION(geoip_time_zone_by_country_and_region)
+{
+	GeoIP * gi;
+	char * country = NULL;
+	char * region = NULL;
+	const char * timezone;
+	int countrylen, arg2len;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|s", &country, &countrylen, &region, &arg2len) == FAILURE) {
+		return;
+	}
+
+	if (!countrylen) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "You need to specify at least the country code.");
+		RETURN_FALSE;
+	}
+	
+	timezone = GeoIP_time_zone_by_country_and_region(country, region);
+	if (timezone == NULL) {
+		RETURN_FALSE;
+	}
+	RETURN_STRING((char*)timezone, 1);
+}
+/* }}} */
+#endif
 
 /*
  * Local variables:
